@@ -39,38 +39,27 @@ extern "C" {
 #endif
 
 #define CONTEXT (SwsContext *)context
+#define BGR32_IS_SWAPPED 1
 
 /**
     \fn swapRGB
 */
-static void swapRGB32(uint32_t w,uint32_t h, uint8_t *to)
+static void swapRGB32(uint32_t w, uint32_t h, uint32_t p, uint8_t *to)
 {
-  uint32_t l=w*h;
-    uint8_t *d=(uint8_t *)to;
-    while(l--)
+    uint32_t i,l;
+    uint8_t *start=(uint8_t *)to;
+    for(i=0; i < h; i++)
     {
-        uint8_t s=d[0];
-        d[0]=d[2];
-        d[2]=s;
-        d+=4;
-        
-    }
-}
-/**
-    \fn swapRGB
-*/
-static void swapRGB24(uint32_t w,uint32_t h, uint8_t *to)
-{
-    uint32_t l=w*h;
-    uint8_t *d=(uint8_t *)to;
-    return;
-
-    while(l--)
-    {
-        uint8_t s=d[0];
-        d[0]=d[2];
-        d[2]=s;
-        d+=3;
+        uint8_t *d=start;
+        start+=p;
+        l=w;
+        while(l--)
+        {
+            uint8_t s=d[0];
+            d[0]=d[2];
+            d[2]=s;
+            d+=4;
+        }
     }
 }
 /**
@@ -89,16 +78,26 @@ static AVPixelFormat ADMColor2LAVColor(ADM_colorspace fromColor_)
     case ADM_COLOR_YUV444: return AV_PIX_FMT_YUV444P;
     case ADM_COLOR_YUV411: return AV_PIX_FMT_YUV411P;
     case ADM_COLOR_YUV422: return AV_PIX_FMT_YUYV422;
+    case ADM_COLOR_UYVY422: return AV_PIX_FMT_UYVY422;
     case ADM_COLOR_YV12: return AV_PIX_FMT_YUV420P;
+    case ADM_COLOR_NV12: return AV_PIX_FMT_NV12;
     case ADM_COLOR_YUV422P: return AV_PIX_FMT_YUV422P;
     case ADM_COLOR_RGB555: return AV_PIX_FMT_RGB555LE;
+    case ADM_COLOR_BGR555: return AV_PIX_FMT_BGR555LE;
     case ADM_COLOR_RGB32A: return AV_PIX_FMT_RGBA;
-    case ADM_COLOR_BGR32A: return AV_PIX_FMT_RGBA; // Faster that way...AV_PIX_FMT_BGR32;
+#ifdef BGR32_IS_SWAPPED
+    case ADM_COLOR_BGR32A: return AV_PIX_FMT_RGBA; // Faster that way...
+#else
+    case ADM_COLOR_BGR32A: return AV_PIX_FMT_BGRA;
+#endif
     case ADM_COLOR_RGB24: return AV_PIX_FMT_RGB24;
     case ADM_COLOR_BGR24: return AV_PIX_FMT_BGR24;
-    case ADM_COLOR_YV12_10BITS: return AV_PIX_FMT_YUV420P10LE;
+    case ADM_COLOR_YUV420_10BITS: return AV_PIX_FMT_YUV420P10LE;
+    case ADM_COLOR_YUV420_12BITS: return AV_PIX_FMT_YUV420P12LE;
     case ADM_COLOR_NV12_10BITS:  return AV_PIX_FMT_P010LE;
     case ADM_COLOR_YUV444_10BITS: return AV_PIX_FMT_YUV444P10LE;
+    case ADM_COLOR_YUV422_10BITS: return AV_PIX_FMT_YUV422P10LE;
+    case ADM_COLOR_YUV444_12BITS: return AV_PIX_FMT_YUV444P12LE;
     case ADM_COLOR_Y8: return AV_PIX_FMT_GRAY8;
     default : ADM_assert(0); 
   }
@@ -129,7 +128,7 @@ uint8_t ADMColorScalerFull::getStrideAndPointers(bool dst,
             srcData[0]=from;
             srcData[1]=NULL;
             srcData[2]=NULL;
-            srcStride[0]=width*2;
+            srcStride[0]=ADM_IMAGE_ALIGN(width*2);
             srcStride[1]=0;
             srcStride[2]=0;
             break;
@@ -138,30 +137,63 @@ uint8_t ADMColorScalerFull::getStrideAndPointers(bool dst,
             srcData[0]=from;
             srcData[1]=NULL;
             srcData[2]=NULL;
-            srcStride[0]=width*3;
+            srcStride[0]=ADM_IMAGE_ALIGN(width*3);
             srcStride[1]=0;
             srcStride[2]=0;
             break;
     case  ADM_COLOR_YV12:
             srcData[0]=from;
-            srcData[1]=from+width*height;
-            srcData[2]=from+((5*width*height)>>2);
+            width=ADM_IMAGE_ALIGN(width);
+            height=ADM_IMAGE_ALIGN(height);
+            from+=width*height;
+            srcData[1]=from;
+            from+=(width>>1)*(height>>1);
+            srcData[2]=from;
             srcStride[0]=width;
             srcStride[1]=width>>1;
             srcStride[2]=width>>1;
             break;
+    case ADM_COLOR_YUV420_10BITS:
+    case ADM_COLOR_YUV420_12BITS:
+            srcData[0]=from;
+            width=ADM_IMAGE_ALIGN(width*2);
+            height=ADM_IMAGE_ALIGN(height);
+            from+=width*height;
+            srcData[1]=from;
+            from+=(width>>1)*(height>>1);
+            srcData[2]=from;
+            srcStride[0]=width;
+            srcStride[1]=width>>1;
+            srcStride[2]=width>>1;
+            break;
+    case ADM_COLOR_NV12:
+            srcData[0]=from;
+            width=ADM_IMAGE_ALIGN(width);
+            height=ADM_IMAGE_ALIGN(height);
+            from+=width*height;
+            srcData[1]=from;
+            srcData[2]=NULL;
+            srcStride[0]=width;
+            srcStride[1]=width;
+            srcStride[2]=0;
+            break;
     case  ADM_COLOR_YUV422:
+    case  ADM_COLOR_UYVY422:        
             srcData[0]=from;
             srcData[1]=NULL;
             srcData[2]=NULL;
-            srcStride[0]=width*2;
+            srcStride[0]=ADM_IMAGE_ALIGN(width*2);
             srcStride[1]=0;
             srcStride[2]=0;
-            break;
+            break;            
     case  ADM_COLOR_YUV422P:
             srcData[0]=from;
-            srcData[1]=from+width*height;
-            srcData[2]=from+((3*width*height)>>1);
+            width=ADM_IMAGE_ALIGN(width);
+            height=ADM_IMAGE_ALIGN(height);
+            from+=width*height;
+            srcData[1]=from;
+            from+=(width>>1)*height;
+            srcData[2]=from;
             srcStride[0]=width;
             srcStride[1]=width>>1;
             srcStride[2]=width>>1;
@@ -171,11 +203,34 @@ uint8_t ADMColorScalerFull::getStrideAndPointers(bool dst,
             srcData[0]=from;
             srcData[1]=NULL;
             srcData[2]=NULL;
-            srcStride[0]=width*4;
+            srcStride[0]=ADM_IMAGE_ALIGN(width*4);
             srcStride[1]=0;
             srcStride[2]=0;
             break;
-    
+    case ADM_COLOR_YUV422_10BITS:
+            srcData[0]=from;
+            width=ADM_IMAGE_ALIGN(width*2);
+            height=ADM_IMAGE_ALIGN(height);
+            from+=width*height;
+            srcData[1]=from;
+            from+=(width>>1)*height;
+            srcData[2]=from;
+            srcStride[0]=width;
+            srcStride[1]=width>>1;
+            srcStride[2]=width>>1;
+            break;
+    case ADM_COLOR_YUV444_12BITS:
+            srcData[0]=from;
+            width=ADM_IMAGE_ALIGN(width*2);
+            height=ADM_IMAGE_ALIGN(height);
+            from+=width*height;
+            srcData[1]=from;
+            from+=width*height;
+            srcData[2]=from;
+            srcStride[0]=width;
+            srcStride[1]=width;
+            srcStride[2]=width;
+            break;
     default:
         ADM_assert(0);
   }
@@ -190,48 +245,67 @@ uint8_t ADMColorScalerFull::getStrideAndPointers(bool dst,
 #define swap16(x) x=((x>>8)&0xff)+(x<<8)
 bool ADMColorScalerFull::convert(uint8_t  *from, uint8_t *to)
 {
-  uint8_t *srcData[3];
-  uint8_t *dstData[3];
-  int srcStride[3];
-  int dstStride[3];
-  
-  getStrideAndPointers(false,from,fromColor,srcData,srcStride);
-  getStrideAndPointers(true,to,toColor,dstData,dstStride);
-  sws_scale(CONTEXT,srcData,srcStride,0,srcHeight,dstData,dstStride);
-  if(toColor==ADM_COLOR_BGR32A)
-  {
-     swapRGB32(dstWidth,dstHeight,to);
-  }
-  if(toColor==ADM_COLOR_BGR24)
-  {
-     swapRGB24(dstWidth,dstHeight,to);
-  }
-  
-  return true;
-  
+    uint8_t *srcData[3];
+    uint8_t *dstData[3];
+    int srcStride[3];
+    int dstStride[3];
+
+    getStrideAndPointers(false,from,fromColor,srcData,srcStride);
+    getStrideAndPointers(true,to,toColor,dstData,dstStride);
+
+    if(fromColor == ADM_COLOR_YV12)
+    {
+        uint8_t *p=srcData[1];
+        srcData[1]=srcData[2];
+        srcData[2]=p;
+    }
+    if(toColor == ADM_COLOR_YV12)
+    {
+        uint8_t *p=dstData[1];
+        dstData[1]=dstData[2];
+        dstData[2]=p;
+    }
+
+#ifdef BGR32_IS_SWAPPED
+    if(fromColor != toColor && fromColor == ADM_COLOR_BGR32A)
+        swapRGB32(srcWidth,srcHeight,srcStride[0],srcData[0]);
+#endif
+    sws_scale(CONTEXT,srcData,srcStride,0,srcHeight,dstData,dstStride);
+#ifdef BGR32_IS_SWAPPED
+    if(fromColor != toColor && toColor==ADM_COLOR_BGR32A)
+        swapRGB32(dstWidth,dstHeight,dstStride[0],dstData[0]);
+#endif
+    return true;
 }
 /**
     \fn convertPlanes
-    \brief Same as convert but the 3 planes are given separately
+    \brief Same as convert but the 3 planes are given separately and the caller takes care of swapping planes
 */
-bool            ADMColorScalerFull::convertPlanes(int  sourceStride[3],int destStride[3],     
-                                  uint8_t   *sourceData[3], uint8_t *destData[3])
+bool ADMColorScalerFull::convertPlanes(int sourceStride[3], int destStride[3], uint8_t *sourceData[3], uint8_t *destData[3])
 {
     int xs[4]={(int)sourceStride[0],(int)sourceStride[1],(int)sourceStride[2],0};
     int xd[4]={(int)destStride[0],(int)destStride[1],(int)destStride[2],0};
     uint8_t *src[4]={NULL,NULL,NULL,NULL};
     uint8_t *dst[4]={NULL,NULL,NULL,NULL};
-     for(int i=0;i<3;i++)
-        {
-            src[i]=sourceData[i];
-            dst[i]=destData[i];
-        }
-     sws_scale(CONTEXT,src,xs,0,srcHeight,dst,xd);
-     return true;
+    for(int i=0;i<3;i++)
+    {
+        src[i]=sourceData[i];
+        dst[i]=destData[i];
+    }
+#ifdef BGR32_IS_SWAPPED
+    if(fromColor != toColor && fromColor == ADM_COLOR_BGR32A)
+        swapRGB32(srcWidth,srcHeight,xs[0],src[0]);
+#endif
+    sws_scale(CONTEXT,src,xs,0,srcHeight,dst,xd);
+#ifdef BGR32_IS_SWAPPED
+    if(fromColor != toColor && toColor == ADM_COLOR_BGR32A)
+        swapRGB32(dstWidth,dstHeight,xd[0],dst[0]);
+#endif
+    return true;
 }
 /**
-    \fn convertPlanes
-    \brief Same as convert but the 3 planes are given separately
+    \fn convertImage
+    \brief Both source and target are given as ADMImage
 */
 bool            ADMColorScalerFull::convertImage(ADMImage *sourceImage, ADMImage *destImage)
 {
@@ -249,8 +323,56 @@ bool            ADMColorScalerFull::convertImage(ADMImage *sourceImage, ADMImage
     
     src[3]=sourceImage->GetReadPtr(PLANAR_ALPHA);
     dst[3]=destImage->GetWritePtr(PLANAR_ALPHA);
-    
+
+    if(fromColor==ADM_COLOR_YV12)
+    {
+        uint8_t *p=src[1];
+        src[1]=src[2];
+        src[2]=p;
+    }
+
+    if(toColor==ADM_COLOR_YV12)
+    {
+        uint8_t *p=dst[1];
+        dst[1]=dst[2];
+        dst[2]=p;
+    }
+    if(fromColor != toColor)
+    {
+        int *itbl = NULL;
+        int *ta = NULL;
+        int sr,dr,bri,con,sat,ret;
+        ret = sws_getColorspaceDetails(CONTEXT,&itbl,&sr,&ta,&dr,&bri,&con,&sat);
+        if(ret < 0)
+        {
+            ADM_warning("Cannot get colorspace details to set color range.\n");
+        }else
+        {
+            sr=(sourceImage->_range == ADM_COL_RANGE_JPEG)? 1 : 0;
+            dr=(destImage->_range   == ADM_COL_RANGE_JPEG)? 1 : 0;
+            ret = sws_setColorspaceDetails(CONTEXT,itbl,sr,ta,dr,bri,con,sat);
+            if(ret < 0)
+            {
+                const char *strSrc = sr? "JPEG" : "MPEG";
+                const char *strDst = dr? "JPEG" : "MPEG";
+                ADM_warning("Cannot set colorspace details, %s --> %s\n",strSrc,strDst);
+            }
+        }
+#ifdef BGR32_IS_SWAPPED
+        if(fromColor == ADM_COLOR_BGR32A)
+            swapRGB32(srcWidth,srcHeight,xs[0],src[0]);
+#endif
+    }else
+    {
+        destImage->_range = sourceImage->_range;
+    }
+
     sws_scale(CONTEXT,src,xs,0,srcHeight,dst,xd);
+
+#ifdef BGR32_IS_SWAPPED
+    if(fromColor != toColor && toColor == ADM_COLOR_BGR32A)
+        swapRGB32(dstWidth,dstHeight,xd[0],dst[0]);
+#endif
     return true;
 }
 
@@ -350,19 +472,37 @@ bool ADMColorScalerFull::convertImage(ADMImage *img, uint8_t *to)
     uint8_t *dstPlanes[3];
     int srcPitch[3];
     int dstPitch[3];
-    int      idstPitch[3];
     img->GetPitches(srcPitch);
     img->GetReadPlanes(srcPlanes);
-    getStrideAndPointers(true,to,toColor, dstPlanes,idstPitch);
-    dstPitch[0]=idstPitch[0];
-    dstPitch[1]=idstPitch[1];
-    dstPitch[2]=idstPitch[2];
-    if(false==convertPlanes(srcPitch,dstPitch,srcPlanes,dstPlanes)) return false;
+    getStrideAndPointers(true,to,toColor, dstPlanes, dstPitch);
 
-    if(toColor==ADM_COLOR_BGR32A)
+    if(fromColor==ADM_COLOR_YV12)
     {
-             swapRGB32(dstWidth,dstHeight,to);
-    }   
-    return true;
+        uint8_t *p=srcPlanes[1];
+        srcPlanes[1]=srcPlanes[2];
+        srcPlanes[2]=p;
+    }
+    if(toColor==ADM_COLOR_YV12)
+    {
+        uint8_t *p=dstPlanes[1];
+        dstPlanes[1]=dstPlanes[2];
+        dstPlanes[2]=p;
+    }
+    if(img->_range==ADM_COL_RANGE_JPEG)
+    {
+        int *itbl = NULL;
+        int *ta = NULL;
+        int sr,dr,bri,con,sat,ret;
+        ret = sws_getColorspaceDetails(CONTEXT,&itbl,&sr,&ta,&dr,&bri,&con,&sat);
+        if(ret < 0)
+            ADM_warning("Cannot get colorspace details to set color range.\n");
+        else
+        {
+            ret = sws_setColorspaceDetails(CONTEXT,itbl,1,ta,0,bri,con,sat);
+            if(ret < 0)
+                ADM_warning("Cannot set colorspace details, JPEG --> MPEG\n");
+        }
+    }
+    return convertPlanes(srcPitch,dstPitch,srcPlanes,dstPlanes);
 }
 //EOF
