@@ -127,11 +127,11 @@ int channels=wavheader.channels;
                                 &max_bytes_output);
     if(!_handle)
     {
-          printf("[FAAC]Cannot open faac with fq=%"PRIu32" chan=%"PRIu32" br=%"PRIu32"\n",
+          printf("[FAAC]Cannot open faac with fq=%" PRIu32" chan=%" PRIu32" br=%" PRIu32"\n",
           wavheader.frequency,channels,_config.bitrate);
           return 0;
     }
-    printf(" [FAAC] : Sample input:%"PRIu32", max byte output%"PRIu32" \n",(uint32_t)samples_input,(uint32_t)max_bytes_output);
+    printf(" [FAAC] : Sample input:%" PRIu32", max byte output%" PRIu32" \n",(uint32_t)samples_input,(uint32_t)max_bytes_output);
     cfg= faacEncGetCurrentConfiguration(_handle);
     
     // Set default conf, same as ffmpeg
@@ -146,7 +146,7 @@ int channels=wavheader.channels;
     cfg->useLfe=0;	
     if (!(ret=faacEncSetConfiguration(_handle, cfg))) 
     {
-        printf("[FAAC] Cannot set conf for faac with fq=%"PRIu32" chan=%"PRIu32" br=%"PRIu32" (err:%d)\n",
+        printf("[FAAC] Cannot set conf for faac with fq=%" PRIu32" chan=%" PRIu32" br=%" PRIu32" (err:%d)\n",
 				wavheader.frequency,channels,_config.bitrate,ret);
 	return 0;
     }
@@ -174,10 +174,10 @@ int channels=wavheader.channels;
     printf("[Faac] Initialized :\n");
     
     printf("[Faac]Version        : %s\n",cfg->name);
-    printf("[Faac]Bitrate        : %"PRIu32"\n",(uint32_t)cfg->bitRate);
+    printf("[Faac]Bitrate        : %" PRIu32"\n",(uint32_t)cfg->bitRate);
     printf("[Faac]Mpeg2 (1)/4(0) : %u\n",cfg->mpegVersion);
     printf("[Faac]Use lfe      ) : %u\n",cfg->useLfe);
-    printf("[Faac]Sample output  : %"PRIu32"\n",_chunk / channels);
+    printf("[Faac]Sample output  : %" PRIu32"\n",_chunk / channels);
     printf("[Faac]Bitrate        : %lu\n",cfg->bitRate*channels);
 
     
@@ -264,7 +264,7 @@ _again:
         return 1;
 }
 #define SZT(x) sizeof(x)/sizeof(diaMenuEntry )
-#define BITRATE(x) {x,QT_TR_NOOP(#x)}
+#define BITRATE(x) {x,QT_TRANSLATE_NOOP("faac",#x)}
 
 /**
     \fn configure
@@ -289,13 +289,13 @@ bool configure (CONFcouple **setup)
                               BITRATE(224),
                               BITRATE(384)
                           };
-    diaElemMenu bitrate(&(config.bitrate),   QT_TR_NOOP("_Bitrate:"), SZT(bitrateM),bitrateM);
+    diaElemMenu bitrate(&(config.bitrate),   QT_TRANSLATE_NOOP("faac","_Bitrate:"), SZT(bitrateM),bitrateM);
   
     
 
     diaElem *elems[]={&bitrate};
     
-    if ( diaFactoryRun(QT_TR_NOOP("Aften Configuration"),1,elems))
+    if ( diaFactoryRun(QT_TRANSLATE_NOOP("faac","Faac Configuration"),1,elems))
     {
         if(*setup) delete *setup;
         *setup=NULL;
