@@ -45,36 +45,50 @@ extern "C"
     }, \
     99, /* Pool Threads : auto */ \
     99, /* Frame Threads : auto */ \
+    0, /* output_bit_depth */ \
     std::string(""), /* Preset */ \
     std::string(""), /* Tuning */ \
     std::string(""), /* Profile */ \
     }, \
     -1, /* Level */ \
-    {1,1}, /* Sar width/height */ \
+    { /* VUI */ \
+        0, /* SAR ID code */ \
+        1,1, /* Sar width/height */ \
+        2, /* uint32_t color_primaries */ \
+        2, /* uint32_t transfer_characteristics */ \
+        2, /* uint32_t matrix_coeffs */ \
+    }, \
     3, /* uint32_t MaxRefFrames */ \
     25, /* uint32_t MinIdr */ \
     250, /* uint32_t MaxIdr */ \
     40, /* uint32_t i_scenecut_threshold */ \
     3, /* uint32_t MaxBFrame */ \
-    1, /* uint32_t i_bframe_adaptative */ \
+    2, /* uint32_t i_bframe_adaptive */ \
     0, /* uint32_t i_bframe_bias */ \
-    2, /* uint32_t i_bframe_pyramid */ \
+    1, /* uint32_t i_bframe_pyramid */ \
     1, /* bool b_deblocking_filter */ \
+    0, /* bool b_open_gop */ \
     0, /* uint32_t interlaced_mode */ \
     false, /* bool constrained_intra */ \
+    true, /* bool b_intra */ \
     40,	/* uint32_t lookahead; */ \
     2, /*    uint32_t weighted_pred */ \
     1, /*    bool weighted_bipred */ \
+    0, /*    bool rect_inter */ \
+    0, /*    bool amp_inter */ \
+    0, /*    bool limit_modes */ \
     0, /*    uint32_t cb_chroma_offset */ \
     0, /*    uint32_t cr_chroma_offset */ \
     3, /*    uint32_t me_method */ \
     16, /*   uint32_t me_range */ \
     5, /*    uint32_t subpel_refine */ \
-    1, /*    uint32_t trellis */ \
+    0x03, /* uint32_t limit_refs */ \
+    3, /*       uint32_t rd_level */ \
     1.0, /*     float psy_rd */ \
+    0, /*       uint32_t rdoq_level */ \
+    0.0, /*     double psy_rdoq */ \
     true, /*    bool fast_pskip */ \
     true, /*    bool dct_decimate */ \
-    0, /*    uint32_t noise_reduction */ \
     0, /*    uint32_t noise_reduction_intra */ \
     0, /*    uint32_t noise_reduction_inter */ \
     true, /*    bool strong_intra_smoothing */ \
@@ -83,7 +97,6 @@ extern "C"
 	0,	/* uint32_t qp_constant; */ \
 	4,	/* uint32_t qp_step; */ \
 	0,	/* uint32_t bitrate; */ \
-	1.0,	/* float rate_tolerance; */ \
 	0,	/* uint32_t vbv_max_bitrate; */ \
 	0,	/* uint32_t vbv_buffer_size; */ \
 	1,	/* uint32_t vbv_buffer_init; */ \
@@ -106,6 +119,7 @@ class x265Encoder : public ADM_coreVideoEncoder
 protected:
                x265_param      param;
                x265_encoder    *handle;
+         const x265_api        *api;
                x265_picture    pic;
                int             plane;
                bool            globalHeader;
