@@ -45,6 +45,7 @@ Ui_chromaShiftWindow::Ui_chromaShiftWindow(QWidget* parent, chromashift *param,A
         memcpy(&(myCrop->param),param,sizeof(chromashift));
         myCrop->_cookie=&ui;
         myCrop->addControl(ui.toolboxLayout);
+        myCrop->setTabOrder();
         myCrop->upload();
         myCrop->sliderChanged();
 
@@ -54,9 +55,7 @@ Ui_chromaShiftWindow::Ui_chromaShiftWindow(QWidget* parent, chromashift *param,A
           SPINNER(U);
           SPINNER(V);
 
-        show();
-        myCrop->adjustCanvasPosition();
-        canvas->parentWidget()->setMinimumSize(30,30); // allow resizing after the dialog has settled
+        setModal(true);
   }
   void Ui_chromaShiftWindow::sliderUpdate(int foo)
   {
@@ -95,6 +94,13 @@ void Ui_chromaShiftWindow::resizeEvent(QResizeEvent *event)
     myCrop->adjustCanvasPosition();
 }
 
+void Ui_chromaShiftWindow::showEvent(QShowEvent *event)
+{
+    QDialog::showEvent(event);
+    myCrop->adjustCanvasPosition();
+    canvas->parentWidget()->setMinimumSize(30,30); // allow resizing after the dialog has settled
+}
+
 #define MYSPIN(x) w->spinBox##x
 //************************
 uint8_t flyChromaShift::upload(void)
@@ -112,6 +118,28 @@ uint8_t flyChromaShift::download(void)
        param.v= MYSPIN(V)->value();
        return true;
 }
+void flyChromaShift::setTabOrder(void)
+{
+    Ui_chromashiftDialog *w=(Ui_chromashiftDialog *)_cookie;
+    std::vector<QWidget *> controls;
+#define PUSH_SPIN(x) controls.push_back(w->spinBox##x);
+    PUSH_SPIN(U)
+    PUSH_SPIN(V)
+
+    controls.insert(controls.end(), buttonList.begin(), buttonList.end());
+    controls.push_back(w->horizontalSlider);
+
+    QWidget *first, *second;
+
+    for(std::vector<QWidget *>::iterator tor = controls.begin(); tor != controls.end(); ++tor)
+    {
+        if(tor+1 == controls.end()) break;
+        first = *tor;
+        second = *(tor+1);
+        _parent->setTabOrder(first,second);
+        //ADM_info("Tab order: %p (%s) --> %p (%s)\n",first,first->objectName().toUtf8().constData(),second,second->objectName().toUtf8().constData());
+    }
+}
 
 /**
       \fn     DIA_getChromaShift
