@@ -1,6 +1,5 @@
 #ifndef Q_crop_h
 #define Q_crop_h
-#include "DIA_flyDialog.h"
 #include "ui_crop.h"
 #include "ADM_image.h"
 #include "DIA_flyDialogQt4.h"
@@ -29,6 +28,9 @@ private slots:
 	void valueChanged(int foo);
 	void autoCrop(bool f);
 	void reset(bool f);
+
+private:
+        void resizeEvent(QResizeEvent *event);
 };
 
 #endif	// Q_crop_h
